#include "jngen.h"

using namespace std;

const int MAX_W = 1000000000;
const int MAX_M = 200000;

using ll = long long;

auto gen(int& n, int& m) {
    int gr = getOpt("g", 0);

    if (gr == 2) {
        m = n - 1;
        auto tree = Tree::random(n).shuffled();
        return tree.edges();
    } else if (gr == 3) {
        auto perm = Array::id(n).shuffled();
        m = n;

        Arrayp edges;
        for (int i = 0; i < n; ++ i) {
            edges.emplace_back(perm[i], perm[(i + 1) % n]);
        }
        return edges;
    } else if (gr == 6) {
        auto perm = Array::id(n).shuffled();
//        cout << perm << "\n";

        int compressnode = 0; // rnd.next(0, n - 1);
        m += compressnode;
        int usenode = min(n + compressnode, m - n);
        n += compressnode;
        int b = rnd.next(1 + compressnode, usenode);

        auto bdi = Array::randomUnique(b - 1, 1, usenode - 1).sort();
        int ttt = bdi.size();
        bdi.push_back(usenode);
        vector<vector<int>> bd; // (b + (n - usenode));
        for (int i = 0, j = 0; i < b; ++i) {
            int cur = i > 0 ? (bdi[i] - bdi[i - 1]) : bdi[i];
            if (cur == 2) {
                ++i;
            }
            if (i == bdi.size()) {
                bd.push_back({perm[j++]});
                bd.push_back({perm[j++]});
                continue;
            }
            bd.emplace_back();
            for (; j < bdi[i]; ++j) {
                bd.back().push_back(perm[j]);
            }
        }
        for (int i = 0; i < n - usenode; ++i) {
            bd.emplace_back();
            bd.back().push_back(perm[n - 1 - i]);
        }
//        std::cout << b << " " << usenode << " " << n - usenode << "\n";

        b = static_cast<int>(bd.size());
        Arrayp edges;
        for (int i = 0; i < b; ++i) {
            ensure(!bd[i].empty());
            ensure(bd[i].size() != 2);
            if (bd[i].size() == 1) { continue; }
            for (int j = 0; j < bd[i].size(); ++j) {
                edges.emplace_back(bd[i][j], bd[i][(j + 1) % bd[i].size()]);
            }
        }

        vector<vector<int>> bdfree = bd;
        for (int i = 0; i < b; ++i) {
            jngen::shuffle(bdfree[i].begin(), bdfree[i].end());
        }

        vector<bool> compressed_nodes(n);
        vector<int> compressed_nodes_to(n);
        iota(compressed_nodes_to.begin(), compressed_nodes_to.end(), -1);

        auto tree = Tree::random(b).shuffle();
        auto treeed = tree.edges();
        auto compress_block_nums = Array::randomUnique(compressnode, 0, treeed.size() - 1);
        set<int> compress_block(compress_block_nums.begin(), compress_block_nums.end());
        for (int i = 0; i < treeed.size(); ++i) {
            if (compress_block.count(i) == 0) {
                // add just edge
                int bx = treeed[i].first;
                int by = treeed[i].second;

                edges.emplace_back(bd[bx][rnd.next(bd[bx].size())], bd[by][rnd.next(bd[by].size())]);
            } else {
                // add compress nodes
            }
        }

        shuffle(edges.begin(), edges.end());

        ensure(edges.size() <= m);
        m = static_cast<int>(edges.size());

        bool printGV = getOpt("gv", false);
        if (printGV) {
            cout << "digraph G {\n";
            for (auto e : edges) {
                cout << "  " << e.first + 1 << " -> " << e.second + 1 << " [arrowhead=none]\n";
            }
            cout << "}\n";
        }

//
//        for (auto& ee : bd) {
//            for (auto t : ee) {
//                std::cout << t <<  " ";
//            }
//            std::cout << "\n";
//        }

        return edges;
    }

    auto g = Graph::random(n, m).connected().g().shuffled();
    auto edges = g.edges();
    return edges;
}

int main(int argc, char *argv[]) {
//#ifdef LOCAL
//    freopen("inf", "w", stdout);
//#endif
    registerGen(argc, argv, 1);
    parseArgs(argc, argv);

    int n = getOpt("n");
    int m = getOpt("m", rnd.next(n - 1, MAX_M));
    m = static_cast<int>(min((ll) m, ((ll) n * ((ll) n - 1)) / 2));
    int w = getOpt("w", MAX_W);

    auto edges = gen(n, m);

    auto ws = Array::randomUnique(m, 1, w);

    printf("%d %d\n", n, m);

    for (int i = 0; i < m; ++i) {
        printf("%d %d %d\n", edges[i].first + 1, edges[i].second + 1, ws[i]);
    }
}