#include "testlib.h"

using namespace std;

int main(int argc, char *argv[]) {

    registerValidation(argc, argv);

    int const N = 200'000;
    int const M = 200'000;
    int const A = 1000'000'000;

    // @formatter:off
    vector<int> const Ng{N, 10, N, N,    N,    N, N, M};
    vector<int> const Mg{M, 15, M, M, 1000, 5000, N, M};
    // @formatter:on

    int __g = validator.group().empty() ? 0 : stoi(validator.group());

    int n = inf.readInt(2, Ng[__g], "n");
    inf.readSpace();
    int m = inf.readInt(1, Mg[__g], "m");
    inf.readEoln();

    set<pair<int, int>> edges;
    vector<vector<int>> g(n);
    set<int> as;
    for (int i = 0; i < m; i++) {
        int u = inf.readInt(1, n, format("u_%d", i + 1));
        inf.readSpace();
        int v = inf.readInt(1, n, format("v_%d", i + 1));
        inf.readSpace();
        int a = inf.readInt(1, A, format("a_%d", i + 1));
        inf.readEoln();

        ensuref(u != v, "graph contains a loop at vertex %d", u + 1);
        ensuref(edges.count({u, v}) == 0, "edge (%d, %d) is repeating", u, v);
        edges.insert({u, v});
        g[u - 1].push_back(v - 1);
        edges.insert({v, u});
        g[v - 1].push_back(u - 1);

        ensuref(as.count(a) == 0, "edge weight %d is repeating on edge no %d", a, i + 1);
        as.insert(a);
    }

    vector<int> vis(n);
    function<void(int)> dfs = [&](int v) {
        vis[v] = 1;
        for (int u : g[v]) {
            if (vis[u]) continue;
            dfs(u);
        }
    };
    dfs(0);
    for (int i = 0; i < n; i++) {
        ensuref(vis[i], "vertex %d is unreachable from 1", i + 1);
    }

    if (__g == 2) ensuref(m == n - 1, "group 2: graph should be a tree");
    if (__g == 3) {
        for (int i = 0; i < n; i++) {
            ensuref(g[i].size() == 2, "group 3: graph should be a cycle but vertex %d has deg != 2", i + 1);
        }
    }
    if (__g == 6) {
        // not checking that sorry not sorry
    }

    inf.readEof();

}