#include "testlib.h"
#include <cstdlib>

using namespace std;

namespace treap {
struct node {
  double y;
  char v;
  int sz = 1;
  node *l, *r;
  node(char v) : v(v), y(rnd.next()), l(nullptr), r(nullptr) {}
};

int size(node *t) {
  if (t == nullptr) {
    return 0;
  }
  return t->sz;
}

void update(node *t) {
  if (t == nullptr) {
    return;
  }
  t->sz = 1 + size(t->l) + size(t->r);
}

node *merge(node *t1, node *t2) {
  if (t1 == nullptr) {
    return t2;
  }
  if (t2 == nullptr) {
    return t1;
  }
  if (t1->y > t2->y) {
    t1->r = merge(t1->r, t2);
    update(t1);
    return t1;
  } else {
    t2->l = merge(t1, t2->l);
    update(t2);
    return t2;
  }
}

std::pair<node *, node *> split(node *t, int x) {
  if (t == nullptr) {
    return make_pair(nullptr, nullptr);
  }
  if (size(t->l) < x) {
    auto [rl, rr] = split(t->r, x - size(t->l) - 1);
    t->r = rl;
    update(t);
    return make_pair(t, rr);
  } else {
    auto [ll, lr] = split(t->l, x);
    t->l = lr;
    update(t);
    return make_pair(ll, t);
  }
}

node *new_node(char v) { return new node(v); }

void to_string(node* t, string & str) {
    if (t) {
        to_string(t->l, str);
        str += t->v;
        to_string(t->r, str);
    }
}

string to_string(node* t) {
    string r;
    to_string(t, r);
    return r;
}
} // namespace treap

int main(int argc, char *argv[]) {

  registerGen(argc, argv, 1);
  int n = opt<int>("n");
  int k = has_opt("k") ? opt<int>("k") : rnd.next(2, n);
  char c = static_cast<char>('a' - 1 + (has_opt("c") ? opt<int>("c") : 6));
  int f = has_opt("f") ? opt<int>("f") : 0;

  string commpon_prefffixs, commpon_prefffixt;
  if (f == 1) {
      int cpl = k >= 6 ? (2 * rnd.next(1, k / 2 - 2)) : 0;
      if (cpl > 0) {
          n -= cpl;
          k -= cpl;
          commpon_prefffixs = rnd.next(format("[a-%c]{%d}", c, cpl - 1));
          commpon_prefffixt = commpon_prefffixs;
          commpon_prefffixt += static_cast<char>(rnd.next(c + 1, 'z'));
          commpon_prefffixs += static_cast<char>(rnd.next('a', c));
      }
  }

  string t = rnd.next(format("[a-%c]{%d}", c, k));

  treap::node *tree = nullptr;

  for (char ch : t) {
    tree = treap::merge(tree, treap::new_node(ch));
  }

  for (int i = 0; i < n - k; ++i) {
      int pos =  2 * rnd.next(1, (k + i) / 2) - 1;
      char nc = static_cast<char>(rnd.next('a', c));
//      cout << pos << "," << nc << " " << treap::to_string(tree) << " ";
      auto [l, r] = treap::split(tree, pos);
      tree = treap::merge(treap::merge(l, treap::new_node(nc)), r);
//      cout << treap::to_string(tree) << "\n";
  }

  println(commpon_prefffixs + treap::to_string(tree));
  println(commpon_prefffixt + t);
}
